@@ -99,9 +99,13 @@ DockView::DockView(Plasma::Corona *corona, QScreen *targetScreen, bool dockWindo
 
         qDebug() << "dock view c++ containment changed 2...";
 
+        //! First load default values from file
+        restoreConfig();
+
+        //! Afterwards override that values in case during creation something different is needed
         setDockWinBehavior(dockWindowBehavior);
 
-        restoreConfig();
+        //! Check the screen assigned to this dock
         reconsiderScreen();
 
         if (!m_visibility) {
@@ -221,6 +225,7 @@ void DockView::init()
     connect(this, &DockView::effectsAreaChanged, this, &DockView::updateEffects);
 
     connect(&m_theme, &Plasma::Theme::themeChanged, this, &DockView::themeChanged);
+    connect(&m_theme, &Plasma::Theme::themeChanged, this, &DockView::themeHasShadowChanged);
 
     connect(this, &DockView::normalThicknessChanged, this, [&]() {
         if (m_behaveAsPlasmaPanel) {
@@ -283,6 +288,10 @@ void DockView::initSignalingForLocationChangeSliding()
                 setBlockAnimations(false);
                 emit showDockAfterLocationChangeFinished();
                 showSettingsWindow();
+
+                if (m_managedLayout) {
+                    m_managedLayout->syncDockViewsToScreens();
+                }
             });
         }
     });
@@ -299,6 +308,10 @@ void DockView::initSignalingForLocationChangeSliding()
                 setBlockAnimations(false);
                 emit showDockAfterScreenChangeFinished();
                 showSettingsWindow();
+
+                if (m_managedLayout) {
+                    m_managedLayout->syncDockViewsToScreens();
+                }
             });
         }
     });
@@ -482,14 +495,9 @@ void DockView::reconsiderScreen()
 
     qDebug() << "dock screen exists  ::: " << screenExists;
 
-    int docksWithTasks = m_managedLayout ? m_managedLayout->noDocksWithTasks() : 0;
-
     //! 1.a primary dock must be always on the primary screen
-    //! 2.the last tasks dock must also always on the primary screen
-    //! even though it has been configured as an explicit
-    if ((m_onPrimary || (tasksPresent() && docksWithTasks == 1 && !screenExists))
-        && (m_screenToFollowId != qGuiApp->primaryScreen()->name()
-            || m_screenToFollow != qGuiApp->primaryScreen())) {
+    if (m_onPrimary && (m_screenToFollowId != qGuiApp->primaryScreen()->name()
+                        || m_screenToFollow != qGuiApp->primaryScreen())) {
         using Plasma::Types;
         QList<Types::Location> edges{Types::BottomEdge, Types::LeftEdge,
                                      Types::TopEdge, Types::RightEdge};
@@ -502,25 +510,18 @@ void DockView::reconsiderScreen()
         qDebug() << "dock location:" << location();
 
         if (edges.contains(location())) {
-            //! case 2
-            if (!m_onPrimary && !screenExists && tasksPresent() && (docksWithTasks == 1)) {
-                qDebug() << "reached case 2 of updating dock primary screen...";
-                setScreenToFollow(qGuiApp->primaryScreen(), false);
-            } else {
-                //! case 1
-                qDebug() << "reached case 1 of updating dock primary screen...";
-                setScreenToFollow(qGuiApp->primaryScreen());
-            }
-
+            //! case 1
+            qDebug() << "reached case 1: of updating dock primary screen...";
+            setScreenToFollow(qGuiApp->primaryScreen());
             syncGeometry();
         }
     } else if (!m_onPrimary) {
-        //! 3.an explicit dock must be always on the correct associated screen
+        //! 2.an explicit dock must be always on the correct associated screen
         //! there are cases that window manager misplaces the dock, this function
         //! ensures that this dock will return at its correct screen
         foreach (auto scr, qGuiApp->screens()) {
             if (scr && scr->name() == m_screenToFollowId) {
-                qDebug() << "updating the explicit screen for dock...";
+                qDebug() << "reached case 2: updating the explicit screen for dock...";
                 setScreenToFollow(scr);
                 syncGeometry();
                 break;
@@ -1292,6 +1293,10 @@ void DockView::setMaskArea(QRect area)
     emit maskAreaChanged();
 }
 
+bool DockView::themeHasShadow() const
+{
+    return PanelShadows::self()->enabled();
+}
 
 QRect DockView::effectsArea() const
 {
@@ -1434,7 +1439,7 @@ void DockView::setManagedLayout(Layout *layout)
         //! Sometimes the activity isnt completely ready, by adding a delay
         //! we try to catch up
         QTimer::singleShot(100, [this]() {
-            if (m_managedLayout) {
+            if (m_managedLayout && m_visibility) {
                 qDebug() << "DOCK VIEW FROM LAYOUT ::: " << m_managedLayout->name() << " - activities: " << m_managedLayout->appliedActivities();
                 applyActivitiesToWindows();
                 emit activitiesChanged();
@@ -1446,7 +1451,7 @@ void DockView::setManagedLayout(Layout *layout)
 
     if (dockCorona->layoutManager()->memoryUsage() == Dock::MultipleLayouts) {
         connectionsManagedLayout[0] = connect(dockCorona->activitiesConsumer(), &KActivities::Consumer::runningActivitiesChanged, this, [&]() {
-            if (m_managedLayout) {
+            if (m_managedLayout && m_visibility) {
                 qDebug() << "DOCK VIEW FROM LAYOUT (runningActivitiesChanged) ::: " << m_managedLayout->name()
                          << " - activities: " << m_managedLayout->appliedActivities();
                 applyActivitiesToWindows();
@@ -1597,7 +1602,10 @@ void DockView::updateEffects()
         }
     } else if (m_behaveAsPlasmaPanel && m_drawEffects) {
         KWindowEffects::enableBlurBehind(winId(), true);
-        KWindowEffects::enableBackgroundContrast(winId(), m_theme.backgroundContrastEnabled(),
+
+        bool drawBackgroundEffect = m_theme.backgroundContrastEnabled() && (m_dockTransparency == 100);
+
+        KWindowEffects::enableBackgroundContrast(winId(), drawBackgroundEffect,
                 m_theme.backgroundContrast(),
                 m_theme.backgroundIntensity(),
                 m_theme.backgroundSaturation());
@@ -1924,8 +1932,8 @@ void DockView::saveConfig()
         return;
 
     auto config = this->containment()->config();
-    config.writeEntry("onPrimary", m_onPrimary);
-    config.writeEntry("dockWindowBehavior", m_dockWinBehavior);
+    config.writeEntry("onPrimary", onPrimary());
+    config.writeEntry("dockWindowBehavior", dockWinBehavior());
     config.sync();
 }
 
@@ -1935,8 +1943,13 @@ void DockView::restoreConfig()
         return;
 
     auto config = this->containment()->config();
-    setOnPrimary(config.readEntry("onPrimary", true));
-    setDockWinBehavior(config.readEntry("dockWindowBehavior", true));
+    m_onPrimary = config.readEntry("onPrimary", true);
+    m_dockWinBehavior = config.readEntry("dockWindowBehavior", true);
+
+    //! Send changed signals at the end in order to be sure that saveConfig
+    //! wont rewrite default/invalid values
+    emit onPrimaryChanged();
+    emit dockWinBehaviorChanged();
 }
 //!END configuration functions
 
