@@ -25,6 +25,7 @@
 #include "screenedgeghostwindow.h"
 #include "../dockcorona.h"
 #include "../layoutmanager.h"
+#include "../screenpool.h"
 #include "../windowinfowrap.h"
 #include "../../liblattedock/extras.h"
 
@@ -439,19 +440,21 @@ void VisibilityManagerPrivate::dodgeActive(WindowId wid)
 
     auto winfo = wm->requestInfo(wid);
 
-    if (!winfo.isValid())
-        return;
-
-    if (!winfo.isActive()) {
-        if (winfo.isPlasmaDesktop())
-            raiseDock(true);
-
+    if (!winfo.isValid() || !winfo.isActive()) {
         winfo = wm->requestInfo(wm->activeWindow());
+
+        if (!winfo.isValid()) {
+            //! very rare case that window manager doesnt have any active window at all
+            raiseDock(true);
+            return;
+        }
     }
 
-    //!dont send false raiseDock signal when containing mouse
-    if (wm->isOnCurrentDesktop(wid) && wm->isOnCurrentActivity(wid)) {
-        raiseDock(!intersects(winfo));
+    //! don't send false raiseDock signal when containing mouse, // Johan comment
+    //! I dont know why that wasnt winfo.wid() //active window, but just wid//the window that made the call
+    if (wm->isOnCurrentDesktop(winfo.wid()) && wm->isOnCurrentActivity(winfo.wid())) {
+        bool overlaps{intersects(winfo)};
+        raiseDock(!overlaps);
     }
 }
 
@@ -468,32 +471,34 @@ void VisibilityManagerPrivate::dodgeMaximized(WindowId wid)
 
     auto winfo = wm->requestInfo(wid);
 
-    if (!winfo.isValid())
-        return;
-
-    if (!winfo.isActive()) {
-        if (winfo.isPlasmaDesktop())
-            raiseDock(true);
-
+    if (!winfo.isValid() || !winfo.isActive()) {
         winfo = wm->requestInfo(wm->activeWindow());
+
+        if (!winfo.isValid()) {
+            //! very rare case that window manager doesnt have any active window at all
+            raiseDock(true);
+            return;
+        }
     }
 
-    auto isMaxVert = [&]() noexcept -> bool {
+    auto intersectsMaxVert = [&]() noexcept -> bool {
         return ((winfo.isMaxVert()
                  || (view->screen() && view->screen()->availableSize().height() <= winfo.geometry().height()))
                 && intersects(winfo));
     };
 
-    auto isMaxHoriz = [&]() noexcept -> bool {
+    auto intersectsMaxHoriz = [&]() noexcept -> bool {
         return ((winfo.isMaxHoriz()
                  || (view->screen() && view->screen()->availableSize().width() <= winfo.geometry().width()))
                 && intersects(winfo));
     };
 
-    //!dont send false raiseDock signal when containing mouse
-    if (wm->isOnCurrentDesktop(wid) && wm->isOnCurrentActivity(wid))
-        raiseDock(view->formFactor() == Plasma::Types::Vertical
-                  ? !isMaxHoriz() : !isMaxVert());
+    //! don't send false raiseDock signal when containing mouse, // Johan comment
+    //! I dont know why that wasnt winfo.wid() //active window, but just wid//the window that made the call
+    if (wm->isOnCurrentDesktop(winfo.wid()) && wm->isOnCurrentActivity(winfo.wid())) {
+        bool overlapsMaximized{view->formFactor() == Plasma::Types::Vertical ? intersectsMaxHoriz() : intersectsMaxVert()};
+        raiseDock(!overlapsMaximized);
+    }
 }
 
 void VisibilityManagerPrivate::dodgeWindows(WindowId wid)
@@ -778,7 +783,8 @@ void VisibilityManagerPrivate::updateAvailableScreenGeometry()
         return;
     }
 
-    QRect tempAvailableScreenGeometry = dockCorona->availableScreenRectWithCriteria(view->containment()->screen(), {Dock::AlwaysVisible}, {});
+    int currentScrId = dockCorona->screenPool()->id(dockView->currentScreen());
+    QRect tempAvailableScreenGeometry = dockCorona->availableScreenRectWithCriteria(currentScrId, {Dock::AlwaysVisible}, {});
 
     if (tempAvailableScreenGeometry != availableScreenGeometry) {
         availableScreenGeometry = tempAvailableScreenGeometry;
@@ -851,24 +857,27 @@ void VisibilityManagerPrivate::updateDynamicBackgroundWindowFlags()
 
     for (const auto &winfo : windows) {
         if (winfo.isValid() && !winfo.isMinimized() && wm->isOnCurrentDesktop(winfo.wid()) && wm->isOnCurrentActivity(winfo.wid())) {
-            if (winfo.isMaximized()
-                && (winfo.geometry().x() == availableScreenGeometry.x())
-                && (winfo.geometry().y() == availableScreenGeometry.y())
-                && (winfo.geometry().bottom() >= availableScreenGeometry.bottom())
-                && (winfo.geometry().right() >= availableScreenGeometry.right())) {
+            if (winfo.isMaximized() && availableScreenGeometry.contains(winfo.geometry().center())) {
+                //! updated implementation to identify the screen that the maximized window is present
+                //! in order to avoid: https://bugs.kde.org/show_bug.cgi?id=397700
                 foundMaximized = true;
             }
 
             bool touchingPanelEdge{false};
 
-            if (view->location() == Plasma::Types::TopEdge) {
-                touchingPanelEdge = (winfo.geometry().y() == availableScreenGeometry.y());
-            } else if (view->location() == Plasma::Types::BottomEdge) {
-                touchingPanelEdge = (winfo.geometry().bottom() == availableScreenGeometry.bottom());
-            } else if (view->location() == Plasma::Types::LeftEdge) {
-                touchingPanelEdge = (winfo.geometry().x() == availableScreenGeometry.x());
-            } else if (view->location() == Plasma::Types::RightEdge) {
-                touchingPanelEdge = (winfo.geometry().right() == availableScreenGeometry.right());
+            QRect screenGeometry = dockView->screenGeometry();
+            bool inCurrentScreen{screenGeometry.contains(winfo.geometry().topLeft()) || screenGeometry.contains(winfo.geometry().bottomRight())};
+
+            if (inCurrentScreen) {
+                if (view->location() == Plasma::Types::TopEdge) {
+                    touchingPanelEdge = (winfo.geometry().y() == availableScreenGeometry.y());
+                } else if (view->location() == Plasma::Types::BottomEdge) {
+                    touchingPanelEdge = (winfo.geometry().bottom() == availableScreenGeometry.bottom());
+                } else if (view->location() == Plasma::Types::LeftEdge) {
+                    touchingPanelEdge = (winfo.geometry().x() == availableScreenGeometry.x());
+                } else if (view->location() == Plasma::Types::RightEdge) {
+                    touchingPanelEdge = (winfo.geometry().right() == availableScreenGeometry.right());
+                }
             }
 
             if (((winfo.isActive() || winfo.isKeepAbove()) && touchingPanelEdge)
